@@ -4,19 +4,18 @@
  * are too numerous to list here. Please refer to the COPYRIGHT
  * file distributed with this source distribution.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
-
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,18 +27,22 @@
 namespace Cine {
 
 extern bool labyrinthCheat;
+extern bool hacksEnabled;
 
 class CineEngine;
 
 class CineConsole : public GUI::Debugger {
 public:
 	CineConsole(CineEngine *vm);
-	virtual ~CineConsole(void);
+	~CineConsole(void) override;
 
 private:
 	CineEngine *_vm;
 
 	bool Cmd_LabyrinthCheat(int argc, const char **argv);
+	bool Cmd_DisableLabyrinthCheat(int argc, const char **argv);
+	bool Cmd_DisableHacks(int argc, const char **argv);
+	bool Cmd_EnableHacks(int argc, const char **argv);
 };
 
 } // End of namespace Cine
