@@ -4,10 +4,10 @@
  * are too numerous to list here. Please refer to the COPYRIGHT
  * file distributed with this source distribution.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,11 +27,14 @@
 #include "common/system.h"
 #include "common/endian.h"
 
+#include "audio/mixer.h"
+
 #include "scumm/imuse/imuse.h"
 #include "scumm/imuse/imuse_internal.h"
 #include "scumm/imuse/instrument.h"
 #include "scumm/resource.h"
 #include "scumm/scumm.h"
+#include "scumm/sound.h"
 
 namespace Scumm {
 
@@ -42,13 +44,14 @@ namespace Scumm {
 //
 ////////////////////////////////////////
 
-IMuseInternal::IMuseInternal() :
-	_native_mt32(false),
-	_enable_gs(false),
-	_isAmiga(false),
-	_midi_adlib(NULL),
-	_midi_native(NULL),
-	_sysex(NULL),
+IMuseInternal::IMuseInternal(ScummEngine *vm, MidiDriverFlags sndType, bool nativeMT32) :
+	_vm(vm),
+	_native_mt32(nativeMT32),
+	_newSystem(vm && vm->_game.id == GID_SAMNMAX),
+	_dynamicChanAllocation(vm && (vm->_game.id != GID_MONKEY2 && vm->_game.id != GID_INDY4)), // For the non-iMuse games that (unfortunately) run on this player we need to pretend we're on the more modern version
+	_midi_adlib(nullptr),
+	_midi_native(nullptr),
+	_sysex(nullptr),
 	_paused(false),
 	_initialized(false),
 	_tempoFactor(0),
@@ -62,9 +65,12 @@ IMuseInternal::IMuseInternal() :
 	_queue_cleared(0),
 	_master_volume(0),
 	_music_volume(0),
+	_music_volume_eff(0),
 	_trigger_count(0),
 	_snm_trigger_index(0),
-	_pcSpeaker(false) {
+	_soundType(sndType),
+	_game_id(vm ? vm->_game.id : 0),
+	_mutex(vm ? vm->_mixer->mutex() : _dummyMutex) {
 	memset(_channel_volume, 0, sizeof(_channel_volume));
 	memset(_channel_volume_eff, 0, sizeof(_channel_volume_eff));
 	memset(_volchan_table, 0, sizeof(_volchan_table));
@@ -76,7 +82,7 @@ IMuseInternal::~IMuseInternal() {
 	// interrupt us, and then do the rest outside
 	// the mutex.
 	{
-		Common::StackLock lock(_mutex, "IMuseInternal::~IMuseInternal()");
+		Common::StackLock lock(_mutex);
 		_initialized = false;
 		stopAllSounds_internal();
 	}
@@ -84,19 +90,13 @@ IMuseInternal::~IMuseInternal() {
 	if (_midi_adlib) {
 		_midi_adlib->close();
 		delete _midi_adlib;
-		_midi_adlib = 0;
+		_midi_adlib = nullptr;
 	}
 
 	if (_midi_native) {
-		if (_native_mt32) {
-			// Reset the MT-32
-			_midi_native->sysEx((const byte *) "\x41\x10\x16\x12\x7f\x00\x00\x01\x00", 9);
-			_system->delayMillis(250);
-		}
-
 		_midi_native->close();
 		delete _midi_native;
-		_midi_native = 0;
+		_midi_native = nullptr;
 	}
 }
 
@@ -112,20 +112,20 @@ byte *IMuseInternal::findStartOfSound(int sound, int ct) {
 
 	byte *ptr = g_scumm->_res->_types[rtSound][sound]._address;
 
-	if (ptr == NULL) {
+	if (ptr == nullptr) {
 		debug(1, "IMuseInternal::findStartOfSound(): Sound %d doesn't exist", sound);
-		return NULL;
+		return nullptr;
 	}
 
 	// Check for old-style headers first, like 'RO'
 	int trFlag = (kMThd | kFORM);
 	if (ptr[0] == 'R' && ptr[1] == 'O' && ptr[2] != 'L')
-		return ct == trFlag ? ptr : 0;
+		return ct == trFlag ? ptr : nullptr;
 	if (ptr[4] == 'S' && ptr[5] == 'O')
-		return ct == trFlag ? ptr + 4 : 0;
+		return ct == trFlag ? ptr + 4 : nullptr;
 
 	ptr += 4;
-	size = READ_BE_UINT32(ptr);
+	//size = READ_BE_UINT32(ptr);
 	ptr += 4;
 
 	// Okay, we're looking for one of those things: either
@@ -143,12 +143,12 @@ byte *IMuseInternal::findStartOfSound(int sound, int ct) {
 	if (ct == (kMThd | kFORM))
 		debug(3, "IMuseInternal::findStartOfSound(): Failed to align on sound %d", sound);
 
-	return 0;
+	return nullptr;
 }
 
 bool IMuseInternal::isMT32(int sound) {
 	byte *ptr = g_scumm->_res->_types[rtSound][sound]._address;
-	if (ptr == NULL)
+	if (ptr == nullptr)
 		return false;
 
 	uint32 tag = READ_BE_UINT32(ptr);
@@ -162,7 +162,7 @@ bool IMuseInternal::isMT32(int sound) {
 		return false;
 
 	case MKTAG('R', 'O', 'L', ' '): // Unfortunately FOA Amiga also uses this resource type
-		return !_isAmiga;
+		return _soundType != MDT_AMIGA && _soundType != MDT_MACINTOSH;
 
 	case MKTAG('M', 'A', 'C', ' '): // Occurs in the Mac version of FOA and MI2
 		return false;
@@ -188,14 +188,14 @@ bool IMuseInternal::isMT32(int sound) {
 	if (ptr[4] == 'S' && ptr[5] == 'O')
 		return false;
 
-	error("Unknown music type: '%c%c%c%c'", (char)tag >> 24, (char)tag >> 16, (char)tag >> 8, (char)tag);
+	error("Unknown music type: '%s'", tag2str(tag));
 
 	return false;
 }
 
 bool IMuseInternal::isMIDI(int sound) {
 	byte *ptr = g_scumm->_res->_types[rtSound][sound]._address;
-	if (ptr == NULL)
+	if (ptr == nullptr)
 		return false;
 
 	uint32 tag = READ_BE_UINT32(ptr);
@@ -230,14 +230,14 @@ bool IMuseInternal::isMIDI(int sound) {
 	if (ptr[4] == 'S' && ptr[5] == 'O')
 		return true;
 
-	error("Unknown music type: '%c%c%c%c'", (char)tag >> 24, (char)tag >> 16, (char)tag >> 8, (char)tag);
+	error("Unknown music type: '%s'", tag2str(tag));
 
 	return false;
 }
 
 bool IMuseInternal::supportsPercussion(int sound) {
 	byte *ptr = g_scumm->_res->_types[rtSound][sound]._address;
-	if (ptr == NULL)
+	if (ptr == nullptr)
 		return false;
 
 	uint32 tag = READ_BE_UINT32(ptr);
@@ -251,7 +251,7 @@ bool IMuseInternal::supportsPercussion(int sound) {
 		return false;
 
 	case MKTAG('R', 'O', 'L', ' '): // Roland LAPC/MT-32/CM32L track, but also used by INDY4 Amiga
-		return !_isAmiga;
+		return _soundType != MDT_AMIGA && _soundType != MDT_MACINTOSH;
 
 	case MKTAG('M', 'A', 'C', ' '): // Occurs in the Mac version of FOA and MI2
 		// This is MIDI, i.e. uses MIDI style program changes, but without a
@@ -274,13 +274,13 @@ bool IMuseInternal::supportsPercussion(int sound) {
 	if (ptr[4] == 'S' && ptr[5] == 'O')
 		return true;
 
-	error("Unknown music type: '%c%c%c%c'", (char)tag >> 24, (char)tag >> 16, (char)tag >> 8, (char)tag);
+	error("Unknown music type: '%s'", tag2str(tag));
 
 	return false;
 }
 
 MidiDriver *IMuseInternal::getBestMidiDriver(int sound) {
-	MidiDriver *driver = NULL;
+	MidiDriver *driver = nullptr;
 
 	if (isMIDI(sound)) {
 		if (_midi_native) {
@@ -296,7 +296,7 @@ MidiDriver *IMuseInternal::getBestMidiDriver(int sound) {
 }
 
 Player *IMuseInternal::allocate_player(byte priority) {
-	Player *player = _players, *best = NULL;
+	Player *player = _players, *best = nullptr;
 	int i;
 	byte bestpri = 255;
 
@@ -313,7 +313,7 @@ Player *IMuseInternal::allocate_player(byte priority) {
 		return best;
 
 	debug(1, "Denying player request");
-	return NULL;
+	return nullptr;
 }
 
 void IMuseInternal::init_players() {
@@ -331,7 +331,7 @@ void IMuseInternal::init_parts() {
 	int i;
 
 	for (i = 0, part = _parts; i != ARRAYSIZE(_parts); i++, part++) {
-		part->init();
+		part->init(_native_mt32);
 		part->_se = this;
 		part->_slot = i;
 	}
@@ -344,24 +344,28 @@ void IMuseInternal::init_parts() {
 ////////////////////////////////////////
 
 void IMuseInternal::on_timer(MidiDriver *midi) {
-	Common::StackLock lock(_mutex, "IMuseInternal::on_timer()");
+	Common::StackLock lock(_mutex);
 	if (_paused || !_initialized)
 		return;
 
+	// CD version only
+	if (_game_id == GID_SAMNMAX && strcmp(_vm->_game.variant, "Floppy"))
+		musicVolumeReduction(midi);
+
 	if (midi == _midi_native || !_midi_native)
 		handleDeferredCommands(midi);
 	sequencer_timers(midi);
 }
 
 void IMuseInternal::pause(bool paused) {
-	Common::StackLock lock(_mutex, "IMuseInternal::pause()");
+	Common::StackLock lock(_mutex);
 	if (_paused == paused)
 		return;
-	int vol = _music_volume;
+	int vol = _music_volume_eff;
 	if (paused)
-		_music_volume = 0;
+		_music_volume_eff = 0;
 	update_volumes();
-	_music_volume = vol;
+	_music_volume_eff = vol;
 
 	// Fix for Bug #1263. The MT-32 apparently fails
 	// sometimes to respond to a channel volume message
@@ -369,7 +373,7 @@ void IMuseInternal::pause(bool paused) {
 	// The result is hanging notes on pause. Reportedly
 	// happens in the original distro, too. To fix that,
 	// just send AllNotesOff to the channels.
-	if (_midi_native && _native_mt32) {
+ 	if (_midi_native && _native_mt32) {
 		for (int i = 0; i < 16; ++i)
 			_midi_native->send(123 << 8 | 0xB0 | i);
 	}
@@ -389,7 +393,7 @@ static void syncWithSerializer(Common::Serializer &s, ImTrigger &it) {
 }
 
 void IMuseInternal::saveLoadIMuse(Common::Serializer &s, ScummEngine *scumm, bool fixAfterLoad) {
-	Common::StackLock lock(_mutex, "IMuseInternal::saveLoadIMuse()");
+	Common::StackLock lock(_mutex);
 
 	int i;
 
@@ -454,17 +458,18 @@ void IMuseInternal::saveLoadIMuse(Common::Serializer &s, ScummEngine *scumm, boo
 }
 
 bool IMuseInternal::get_sound_active(int sound) const {
-	Common::StackLock lock(_mutex, "IMuseInternal::get_sound_active()");
+	Common::StackLock lock(_mutex);
 	return getSoundStatus_internal(sound, false) != 0;
 }
 
 int32 IMuseInternal::doCommand(int numargs, int a[]) {
-	Common::StackLock lock(_mutex, "IMuseInternal::doCommand()");
+	Common::StackLock lock(_mutex);
 	return doCommand_internal(numargs, a);
 }
 
+
 uint32 IMuseInternal::property(int prop, uint32 value) {
-	Common::StackLock lock(_mutex, "IMuseInternal::property()");
+	Common::StackLock lock(_mutex);
 	switch (prop) {
 	case IMuse::PROP_TEMPO_BASE:
 		// This is a specified as a percentage of normal
@@ -474,27 +479,6 @@ uint32 IMuseInternal::property(int prop, uint32 value) {
 			_tempoFactor = value;
 		break;
 
-	case IMuse::PROP_NATIVE_MT32:
-		_native_mt32 = (value > 0);
-		Instrument::nativeMT32(_native_mt32);
-		if (_midi_native && _native_mt32)
-			initMT32(_midi_native);
-		break;
-
-	case IMuse::PROP_GS:
-		_enable_gs = (value > 0);
-
-		// GS Mode emulates MT-32 on a GS device, so _native_mt32 should always be true
-		if (_midi_native && _enable_gs) {
-			_native_mt32 = true;
-			initGM(_midi_native);
-		}
-		break;
-
-	case IMuse::PROP_AMIGA:
-		_isAmiga = (value > 0);
-		break;
-
 	case IMuse::PROP_LIMIT_PLAYERS:
 		if (value > 0 && value <= ARRAYSIZE(_players))
 			_player_limit = (int)value;
@@ -504,14 +488,6 @@ uint32 IMuseInternal::property(int prop, uint32 value) {
 		_recycle_players = (value != 0);
 		break;
 
-	case IMuse::PROP_GAME_ID:
-		_game_id = value;
-		break;
-
-	case IMuse::PROP_PC_SPEAKER:
-		_pcSpeaker = (value != 0);
-		break;
-
 	default:
 		break;
 	}
@@ -522,12 +498,12 @@ uint32 IMuseInternal::property(int prop, uint32 value) {
 void IMuseInternal::addSysexHandler(byte mfgID, sysexfunc handler) {
 	// TODO: Eventually support multiple sysEx handlers and pay
 	// attention to the client-supplied manufacturer ID.
-	Common::StackLock lock(_mutex, "IMuseInternal::property()");
+	Common::StackLock lock(_mutex);
 	_sysex = handler;
 }
 
 void IMuseInternal::startSoundWithNoteOffset(int sound, int offset) {
-	Common::StackLock lock(_mutex, "IMuseInternal::startSound()");
+	Common::StackLock lock(_mutex);
 	startSound_internal(sound, offset);
 }
 
@@ -538,13 +514,16 @@ void IMuseInternal::startSoundWithNoteOffset(int sound, int offset) {
 ////////////////////////////////////////
 
 void IMuseInternal::setMusicVolume(int vol) {
-	Common::StackLock lock(_mutex, "IMuseInternal::setMusicVolume()");
+	Common::StackLock lock(_mutex);
 	if (vol > 255)
 		vol = 255;
 	if (_music_volume == vol)
 		return;
+
 	_music_volume = vol;
-	vol = _master_volume * _music_volume / 255;
+	_music_volume_eff = _music_volume;
+
+	vol = _master_volume * _music_volume_eff / 255;
 	for (uint i = 0; i < ARRAYSIZE(_channel_volume); i++) {
 		_channel_volume_eff[i] = _channel_volume[i] * vol / 255;
 	}
@@ -553,27 +532,27 @@ void IMuseInternal::setMusicVolume(int vol) {
 }
 
 void IMuseInternal::startSound(int sound) {
-	Common::StackLock lock(_mutex, "IMuseInternal::startSound()");
+	Common::StackLock lock(_mutex);
 	startSound_internal(sound);
 }
 
 void IMuseInternal::stopSound(int sound) {
-	Common::StackLock lock(_mutex, "IMuseInternal::stopSound()");
+	Common::StackLock lock(_mutex);
 	stopSound_internal(sound);
 }
 
 void IMuseInternal::stopAllSounds() {
-	Common::StackLock lock(_mutex, "IMuseInternal::stopAllSounds()");
+	Common::StackLock lock(_mutex);
 	stopAllSounds_internal();
 }
 
 int IMuseInternal::getSoundStatus(int sound) const {
-	Common::StackLock lock(_mutex, "IMuseInternal::getSoundStatus()");
+	Common::StackLock lock(_mutex);
 	return getSoundStatus_internal(sound, true);
 }
 
 int IMuseInternal::getMusicTimer() {
-	Common::StackLock lock(_mutex, "IMuseInternal::getMusicTimer()");
+	Common::StackLock lock(_mutex);
 	int best_time = 0;
 	const Player *player = _players;
 	for (int i = ARRAYSIZE(_players); i; i--, player++) {
@@ -672,7 +651,7 @@ bool IMuseInternal::startSound_internal(int sound, int offset) {
 	if (_game_id == GID_SAMNMAX && sound == 82 && getSoundStatus_internal(81, false))
 		ImClearTrigger(81, 1);
 
-	// Workaround for monkey2 bug #1410 / Scabb Island
+	// WORKAROUND for monkey2 bug #1410 / Scabb Island
 	//
 	// Tunes involved:
 	// 100 - Captain Dread's map
@@ -689,11 +668,11 @@ bool IMuseInternal::startSound_internal(int sound, int offset) {
 	if (_game_id == GID_MONKEY2 && (sound == 107) && (getSoundStatus_internal(100, true) == 1))
 		return false;
 
-	// In some cases 107 is running and doesn't get killed at Dread's map
+	// WORKAROUND: In some cases 107 is running and doesn't get killed at Dread's map
 	if (_game_id == GID_MONKEY2 && (sound == 100) && (getSoundStatus_internal(107, true) == 1))
 		IMuseInternal::stopSound_internal(107);
 
-	// Workaround for monkey2 bug #1410 / Booty Island
+	// WORKAROUND for monkey2 bug #1410 / Booty Island
 	//
 	// Tunes involved
 	// 100 - Captain Dread's map
@@ -736,6 +715,18 @@ int IMuseInternal::stopAllSounds_internal() {
 }
 
 int IMuseInternal::getSoundStatus_internal(int sound, bool ignoreFadeouts) const {
+	if (_game_id != GID_MONKEY2) {
+		// The whole fadeout checking / ignoring is not present in any of the original
+		// drivers, but necessary as a WORKAROUND for a bug in Monkey Island 2 that also
+		// happens with the original interpreter (bug no. 385: "No music if room
+		// transition is too fast"). The bug is caused by sloppy scripting, but probably
+		// wouldn't ever be seen on machines of that era, when the loading time for a
+		// room change would take longer than the fadeout time.
+		// Since the code is objectively wrong and the workaround is not known to be
+		// needed elsewhere, we restrict it to Monkey Island 2.
+		ignoreFadeouts = false;
+	}
+
 	const Player *player = _players;
 	for (int i = ARRAYSIZE(_players); i; i--, player++) {
 		if (player->isActive() && (!ignoreFadeouts || !player->isFadingOut())) {
@@ -768,7 +759,7 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 	int i;
 	byte cmd = a[0] & 0xFF;
 	byte param = a[0] >> 8;
-	Player *player = NULL;
+	Player *player = nullptr;
 
 	if (!_initialized && (cmd || param))
 		return -1;
@@ -804,7 +795,7 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 		case 12:
 			// Sam & Max: Player-scope commands
 			player = findActivePlayer(a[1]);
-			if (player == NULL)
+			if (player == nullptr)
 				return -1;
 
 			switch (a[3]) {
@@ -884,14 +875,20 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 			error("doCommand(%d [%d/%d], %d, %d, %d, %d, %d, %d, %d) unsupported", a[0], param, cmd, a[1], a[2], a[3], a[4], a[5], a[6], a[7]);
 		}
 	} else if (param == 1) {
+		Part *part = nullptr;
 		if ((1 << cmd) & 0x783FFF) {
 			player = findActivePlayer(a[1]);
 			if (!player)
 				return -1;
-			if ((1 << cmd) & (1 << 11 | 1 << 22)) {
+			if (_newSystem && cmd == 5) {
+				assert(a[3] >= 0 && a[3] <= 15);
+				part = player->getPart(a[2]);
+				if (!part)
+					return -1;
+			} else if (((1 << cmd) & (1 << 11 | 1 << 22))) {
 				assert(a[2] >= 0 && a[2] <= 15);
-				player = (Player *)player->getPart(a[2]);
-				if (!player)
+				part = player->getPart(a[2]);
+				if (!part)
 					return -1;
 			}
 		}
@@ -941,12 +938,20 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 		case 2:
 			return player->setVolume(a[2]);
 		case 3:
-			player->setPan(a[2]);
+			if (_newSystem)
+				player->setSpeed(a[3]);
+			else
+				player->setPan(a[2]);
 			return 0;
 		case 4:
 			return player->setTranspose(a[2], a[3]);
 		case 5:
-			player->setDetune(a[2]);
+			if (_newSystem) {
+				assert(part);
+				part->volControlSensitivity(a[4]);
+			} else {
+				player->setDetune(a[2]);
+			}
 			return 0;
 		case 6:
 			// WORKAROUND for bug #2242. When playing the
@@ -977,7 +982,8 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 			player->clearLoop();
 			return 0;
 		case 11:
-			((Part *)player)->set_onoff(a[3] != 0);
+			assert(part);
+			part->set_onoff(a[3] != 0);
 			return 0;
 		case 12:
 			return player->setHook(a[2], a[3], a[4]);
@@ -996,7 +1002,8 @@ int32 IMuseInternal::doCommand_internal(int numargs, int a[]) {
 		case 21:
 			return -1;
 		case 22:
-			((Part *)player)->volume(a[3]);
+			assert(part);
+			part->volume(a[3]);
 			return 0;
 		case 23:
 			return query_queue(a[1]);
@@ -1045,11 +1052,11 @@ void IMuseInternal::handle_marker(uint id, byte data) {
 int IMuseInternal::get_channel_volume(uint a) {
 	if (a < 8)
 		return _channel_volume_eff[a];
-	return (_master_volume * _music_volume / 255) / 2;
+	return (_master_volume * _music_volume_eff / 255) / 2;
 }
 
 Part *IMuseInternal::allocate_part(byte pri, MidiDriver *midi) {
-	Part *part, *best = NULL;
+	Part *part, *best = nullptr;
 	int i;
 
 	for (i = ARRAYSIZE(_parts), part = _parts; i != 0; i--, part++) {
@@ -1113,9 +1120,9 @@ int IMuseInternal::set_volchan(int sound, int volchan) {
 		}
 		return -1;
 	} else {
-		best = NULL;
+		best = nullptr;
 		num = 0;
-		sameid = NULL;
+		sameid = nullptr;
 		for (i = ARRAYSIZE(_players), player = _players; i != 0; i--, player++) {
 			if (player->isActive()) {
 				if (player->_vol_chan == (uint16)volchan) {
@@ -1127,12 +1134,12 @@ int IMuseInternal::set_volchan(int sound, int volchan) {
 				}
 			}
 		}
-		if (sameid == NULL)
+		if (sameid == nullptr)
 			return -1;
-		if (num >= r)
+		if (best != nullptr && num >= r)
 			best->clear();
-		player->_vol_chan = volchan;
-		player->setVolume(player->getVolume());
+		sameid->_vol_chan = volchan;
+		sameid->setVolume(sameid->getVolume());
 		return 0;
 	}
 }
@@ -1205,7 +1212,7 @@ int IMuseInternal::setImuseMasterVolume(uint vol) {
 	if (_master_volume == vol)
 		return 0;
 	_master_volume = vol;
-	vol = _master_volume * _music_volume / 255;
+	vol = _master_volume * _music_volume_eff / 255;
 	for (uint i = 0; i < ARRAYSIZE(_channel_volume); i++) {
 		_channel_volume_eff[i] = _channel_volume[i] * vol / 255;
 	}
@@ -1246,7 +1253,7 @@ int32 IMuseInternal::ImSetTrigger(int sound, int id, int a, int b, int c, int d,
 	// represented by MIDI SysEx block 00 xx(F7)
 	// where "xx" is the marker ID.
 	uint16 oldest_trigger = 0;
-	ImTrigger *oldest_ptr = NULL;
+	ImTrigger *oldest_ptr = nullptr;
 
 	int i;
 	ImTrigger *trig = _snm_triggers;
@@ -1330,7 +1337,7 @@ int IMuseInternal::set_channel_volume(uint chan, uint vol) {
 		return -1;
 
 	_channel_volume[chan] = vol;
-	_channel_volume_eff[chan] = _master_volume * _music_volume * vol / 255 / 255;
+	_channel_volume_eff[chan] = _master_volume * _music_volume_eff * vol / 255 / 255;
 	update_volumes();
 	return 0;
 }
@@ -1345,6 +1352,40 @@ void IMuseInternal::update_volumes() {
 	}
 }
 
+void IMuseInternal::musicVolumeReduction(MidiDriver *midi) {
+	int curVol;
+	int curEffVol;
+	int factor = 2; // The music volume variables are 0-255, and we need 0-127
+
+	if (_paused)
+		return;
+
+	_musicVolumeReductionTimer += midi->getBaseTempo();
+	while (_musicVolumeReductionTimer >= MUS_REDUCTION_TIMER_TICKS) {
+		_musicVolumeReductionTimer -= MUS_REDUCTION_TIMER_TICKS;
+		curVol = _music_volume / factor;
+
+		if (_vm->_sound->speechIsPlaying())
+			curVol = (90 * curVol) >> 7;
+
+		curEffVol = _music_volume_eff / factor;
+
+		// The reduction curve is pretty slow, but running
+		// the original through a debugger shows the same behavior...
+		if (curEffVol > curVol)
+			_music_volume_eff = (curEffVol - 1) * factor;
+
+		if (curEffVol < curVol)
+			_music_volume_eff = (curEffVol + 1) * factor;
+	}
+
+	for (uint i = 0; i < ARRAYSIZE(_channel_volume); i++) {
+		_channel_volume_eff[i] = _channel_volume[i] * (_master_volume * _music_volume_eff / 255) / 255;
+	}
+
+	update_volumes();
+}
+
 int IMuseInternal::set_volchan_entry(uint a, uint b) {
 	if (a >= 8)
 		return -1;
@@ -1420,7 +1461,7 @@ Player *IMuseInternal::findActivePlayer(int id) {
 		if (player->isActive() && player->getID() == (uint16)id)
 			return player;
 	}
-	return NULL;
+	return nullptr;
 }
 
 int IMuseInternal::get_volchan_entry(uint a) {
@@ -1429,24 +1470,23 @@ int IMuseInternal::get_volchan_entry(uint a) {
 	return -1;
 }
 
-IMuseInternal *IMuseInternal::create(OSystem *syst, MidiDriver *nativeMidiDriver, MidiDriver *adlibMidiDriver) {
-	IMuseInternal *i = new IMuseInternal;
-	i->initialize(syst, nativeMidiDriver, adlibMidiDriver);
+IMuseInternal *IMuseInternal::create(ScummEngine *vm, MidiDriver *nativeMidiDriver, MidiDriver *adlibMidiDriver, MidiDriverFlags sndType, bool nativeMT32) {
+	IMuseInternal *i = new IMuseInternal(vm, sndType, nativeMT32);
+	i->initialize(vm->_system, nativeMidiDriver, adlibMidiDriver);
 	return i;
 }
 
 int IMuseInternal::initialize(OSystem *syst, MidiDriver *native_midi, MidiDriver *adlib_midi) {
 	int i;
 
-	_system = syst;
 	_midi_native = native_midi;
 	_midi_adlib = adlib_midi;
-	if (native_midi != NULL) {
+	if (native_midi != nullptr) {
 		_timer_info_native.imuse = this;
 		_timer_info_native.driver = native_midi;
 		initMidiDriver(&_timer_info_native);
 	}
-	if (adlib_midi != NULL) {
+	if (adlib_midi != nullptr) {
 		_timer_info_adlib.imuse = this;
 		_timer_info_adlib.driver = adlib_midi;
 		initMidiDriver(&_timer_info_adlib);
@@ -1478,149 +1518,6 @@ void IMuseInternal::initMidiDriver(TimerCallbackInfo *info) {
 	info->driver->setTimerCallback(info, &IMuseInternal::midiTimerCallback);
 }
 
-void IMuseInternal::initMT32(MidiDriver *midi) {
-	byte buffer[52];
-
-	// Reset the MT-32
-	midi->sysEx((const byte *) "\x41\x10\x16\x12\x7f\x00\x00\x01\x00", 9);
-	_system->delayMillis(250);
-
-	// Setup master tune, reverb mode, reverb time, reverb level,
-	// channel mapping, partial reserve and master volume
-	midi->sysEx((const byte *) "\x41\x10\x16\x12\x10\x00\x00\x40\x00\x04\x04\x04\x04\x04\x04\x04\x04\x04\x04\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x64\x77", 31);
-	_system->delayMillis(250);
-
-	// Map percussion to notes 24 - 34 without reverb
-	midi->sysEx((const byte *) "\x41\x10\x16\x12\x03\x01\x10\x40\x64\x07\x00\x4a\x64\x06\x00\x41\x64\x07\x00\x4b\x64\x08\x00\x45\x64\x06\x00\x44\x64\x0b\x00\x51\x64\x05\x00\x43\x64\x08\x00\x50\x64\x07\x00\x42\x64\x03\x00\x4c\x64\x07\x00\x44", 52);
-	_system->delayMillis(250);
-
-	// Compute version string (truncated to 20 chars max.)
-	Common::String infoStr = "ScummVM ";
-	infoStr += gScummVMVersion;
-	int len = infoStr.size();
-	if (len > 20)
-		len = 20;
-
-	// Display a welcome message on MT-32 displays.
-	memcpy(&buffer[0], "\x41\x10\x16\x12\x20\x00\x00", 7);
-	memcpy(&buffer[7], "                    ", 20);
-	memcpy(buffer + 7 + (20 - len) / 2, infoStr.c_str(), len);
-	byte checksum = 0;
-	for (int i = 4; i < 27; ++i)
-		checksum -= buffer[i];
-	buffer[27] = checksum & 0x7F;
-	midi->sysEx(buffer, 28);
-	_system->delayMillis(1000);
-}
-
-void IMuseInternal::initGM(MidiDriver *midi) {
-	byte buffer[12];
-	int i;
-
-	// General MIDI System On message
-	// Resets all GM devices to default settings
-	memcpy(&buffer[0], "\x7E\x7F\x09\x01", 4);
-	midi->sysEx(buffer, 4);
-	debug(2, "GM SysEx: GM System On");
-	_system->delayMillis(200);
-
-	if (_enable_gs) {
-		// All GS devices recognize the GS Reset command,
-		// even using Roland's ID. It is impractical to
-		// support other manufacturers' devices for
-		// further GS settings, as there are limitless
-		// numbers of them out there that would each
-		// require individual SysEx commands with unique IDs.
-
-		// Roland GS SysEx ID
-		memcpy(&buffer[0], "\x41\x10\x42\x12", 4);
-
-		// GS Reset
-		memcpy(&buffer[4], "\x40\x00\x7F\x00\x41", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: GS Reset");
-		_system->delayMillis(200);
-
-		// Set global Master Tune to 442.0kHz, as on the MT-32
-		memcpy(&buffer[4], "\x40\x00\x00\x00\x04\x04\x0F\x29", 8);
-		midi->sysEx(buffer, 12);
-		debug(2, "GS SysEx: Master Tune set to 442.0kHz");
-
-		// Note: All Roland GS devices support CM-64/32L maps
-
-		// Set Channels 1-16 to SC-55 Map, then CM-64/32L Variation
-		for (i = 0; i < 16; ++i) {
-			midi->send((127 << 16) | (0  << 8) | (0xB0 | i));
-			midi->send((1   << 16) | (32 << 8) | (0xB0 | i));
-			midi->send((0   << 16) | (0  << 8) | (0xC0 | i));
-		}
-		debug(2, "GS Program Change: CM-64/32L Map Selected");
-
-		// Set Percussion Channel to SC-55 Map (CC#32, 01H), then
-		// Switch Drum Map to CM-64/32L (MT-32 Compatible Drums)
-		midi->getPercussionChannel()->controlChange(0, 0);
-		midi->getPercussionChannel()->controlChange(32, 1);
-		midi->send(127 << 8 | 0xC0 | 9);
-		debug(2, "GS Program Change: Drum Map is CM-64/32L");
-
-		// Set Master Chorus to 0. The MT-32 has no chorus capability.
-		memcpy(&buffer[4], "\x40\x01\x3A\x00\x05", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: Master Chorus Level is 0");
-
-		// Set Channels 1-16 Reverb to 64, which is the
-		// equivalent of MT-32 default Reverb Level 5
-		for (i = 0; i < 16; ++i)
-			midi->send((64 << 16) | (91 << 8) | (0xB0 | i));
-		debug(2, "GM Controller 91 Change: Channels 1-16 Reverb Level is 64");
-
-		// Set Channels 1-16 Pitch Bend Sensitivity to
-		// 12 semitones; then lock the RPN by setting null.
-		for (i = 0; i < 16; ++i) {
-			midi->setPitchBendRange(i, 12);
-		}
-		debug(2, "GM Controller 6 Change: Channels 1-16 Pitch Bend Sensitivity is 12 semitones");
-
-		// Set channels 1-16 Mod. LFO1 Pitch Depth to 4
-		memcpy(&buffer[4], "\x40\x20\x04\x04\x18", 5);
-		for (i = 0; i < 16; ++i) {
-			buffer[5] = 0x20 + i;
-			buffer[8] = 0x18 - i;
-			midi->sysEx(buffer, 9);
-		}
-		debug(2, "GS SysEx: Channels 1-16 Mod. LFO1 Pitch Depth Level is 4");
-
-		// Set Percussion Channel Expression to 80
-		midi->getPercussionChannel()->controlChange(11, 80);
-		debug(2, "GM Controller 11 Change: Percussion Channel Expression Level is 80");
-
-		// Turn off Percussion Channel Rx. Expression so that
-		// Expression cannot be modified. I don't know why, but
-		// Roland does it this way.
-		memcpy(&buffer[4], "\x40\x10\x0E\x00\x22", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: Percussion Channel Rx. Expression is OFF");
-
-		// Change Reverb Character to 0. I don't think this
-		// sounds most like MT-32, but apparently Roland does.
-		memcpy(&buffer[4], "\x40\x01\x31\x00\x0E", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: Reverb Character is 0");
-
-		// Change Reverb Pre-LF to 4, which is similar to
-		// what MT-32 reverb does.
-		memcpy(&buffer[4], "\x40\x01\x32\x04\x09", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: Reverb Pre-LF is 4");
-
-		// Change Reverb Time to 106; the decay on Hall 2
-		// Reverb is too fast compared to the MT-32's
-		memcpy(&buffer[4], "\x40\x01\x34\x6A\x21", 5);
-		midi->sysEx(buffer, 9);
-		debug(2, "GS SysEx: Reverb Time is 106");
-	}
-}
-
 void IMuseInternal::init_queue() {
 	_queue_adding = false;
 	_queue_pos = 0;
@@ -1701,15 +1598,76 @@ void IMuseInternal::midiTimerCallback(void *data) {
 	info->imuse->on_timer(info->driver);
 }
 
+MidiChannel *IMuseInternal::allocateChannel(MidiDriver *midi, byte prio) {
+	MidiChannel *mc = midi->allocateChannel();
+	if (mc)
+		return mc;
+
+	Part *best = nullptr;
+	for (Part *part = _parts; part < &_parts[ARRAYSIZE(_parts)]; ++part) {
+		if (!part->_percussion && part->_mc && part->_mc->device() == midi && part->_pri_eff <= prio) {
+			prio = part->_pri_eff;
+			best = part;
+		}
+	}
+
+	if (best) {
+		best->off();
+		suspendPart(best);
+		mc = midi->allocateChannel();
+	}
+
+	return mc;
+}
+
+bool IMuseInternal::reassignChannelAndResumePart(MidiChannel *mc) {
+	while (!_waitingPartsQueue.empty()) {
+		Part *part = _waitingPartsQueue.remove_at(0);
+		if (part->_player) {
+			part->_mc = mc;
+			part->sendAll();
+			return true;
+		}
+	}
+
+	return false;
+}
+
+void IMuseInternal::suspendPart(Part *part) {
+	if (_waitingPartsQueue.empty()) {
+		_waitingPartsQueue.push_back(part);
+		return;
+	}
+
+	for (Common::Array<Part*>::iterator it = _waitingPartsQueue.begin(); it != _waitingPartsQueue.end(); ++it) {
+		if ((*it)->_pri_eff > part->_pri_eff)
+			continue;
+		_waitingPartsQueue.insert(it, part);
+		return;
+	}
+}
+
+void IMuseInternal::removeSuspendedPart(Part *part) {
+	for (Common::Array<Part*>::iterator it = _waitingPartsQueue.begin(); it != _waitingPartsQueue.end(); ++it) {
+		if (*it != part)
+			continue;
+		_waitingPartsQueue.erase(it);
+		return;
+	}
+}
+
 void IMuseInternal::reallocateMidiChannels(MidiDriver *midi) {
 	Part *part, *hipart;
 	int i;
 	byte hipri, lopri;
 	Part *lopart;
 
+	if (!_dynamicChanAllocation)
+		return;
+
 	while (true) {
 		hipri = 0;
-		hipart = NULL;
+		hipart = nullptr;
 		for (i = 32, part = _parts; i; i--, part++) {
 			if (part->_player && part->_player->getMidiDriver() == midi &&
 			        !part->_percussion && part->_on &&
@@ -1722,9 +1680,9 @@ void IMuseInternal::reallocateMidiChannels(MidiDriver *midi) {
 		if (!hipart)
 			return;
 
-		if ((hipart->_mc = midi->allocateChannel()) == NULL) {
+		if ((hipart->_mc = midi->allocateChannel()) == nullptr) {
 			lopri = 255;
-			lopart = NULL;
+			lopart = nullptr;
 			for (i = 32, part = _parts; i; i--, part++) {
 				if (part->_mc && part->_mc->device() == midi && part->_pri_eff <= lopri) {
 					lopri = part->_pri_eff;
@@ -1732,11 +1690,11 @@ void IMuseInternal::reallocateMidiChannels(MidiDriver *midi) {
 				}
 			}
 
-			if (lopart == NULL || lopri >= hipri)
+			if (lopart == nullptr || lopri >= hipri)
 				return;
 			lopart->off();
 
-			if ((hipart->_mc = midi->allocateChannel()) == NULL)
+			if ((hipart->_mc = midi->allocateChannel()) == nullptr)
 				return;
 		}
 		hipart->sendAll();
@@ -1745,7 +1703,7 @@ void IMuseInternal::reallocateMidiChannels(MidiDriver *midi) {
 
 void IMuseInternal::setGlobalInstrument(byte slot, byte *data) {
 	if (slot < 32) {
-		if (_pcSpeaker)
+		if (_soundType == MDT_PCSPK)
 			_global_instruments[slot].pcspk(data);
 		else
 			_global_instruments[slot].adlib(data);
@@ -1763,9 +1721,9 @@ void IMuseInternal::copyGlobalInstrument(byte slot, Instrument *dest) {
 	const byte defaultInstr[30] = { 0 };
 
 	if (_global_instruments[slot].isValid()) {
-		// In case we have an valid instrument set up, copy it to the part.
+		// In case we have a valid instrument set up, copy it to the part.
 		_global_instruments[slot].copy_to(dest);
-	} else if (_pcSpeaker) {
+	} else if (_soundType == MDT_PCSPK) {
 		debug(0, "Trying to use non-existent global PC Speaker instrument %d", slot);
 		dest->pcspk(defaultInstr);
 	} else {
@@ -1785,8 +1743,8 @@ void IMuseInternal::copyGlobalInstrument(byte slot, Instrument *dest) {
  * of the implementation to be changed and updated
  * without requiring a recompile of the client code.
  */
-IMuse *IMuse::create(OSystem *syst, MidiDriver *nativeMidiDriver, MidiDriver *adlibMidiDriver) {
-	IMuseInternal *engine = IMuseInternal::create(syst, nativeMidiDriver, adlibMidiDriver);
+IMuse *IMuse::create(ScummEngine *vm, MidiDriver *nativeMidiDriver, MidiDriver *adlibMidiDriver, MidiDriverFlags sndType, bool nativeMT32) {
+	IMuseInternal *engine = IMuseInternal::create(vm, nativeMidiDriver, adlibMidiDriver, sndType, nativeMT32);
 	return engine;
 }
 
