@@ -4,10 +4,10 @@
  * are too numerous to list here. Please refer to the COPYRIGHT
  * file distributed with this source distribution.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
  */
 
 #include "common/config-manager.h"
@@ -26,7 +26,7 @@
 #include "common/ptr.h"
 #include "common/textconsole.h"
 
-#include "graphics/palette.h"
+#include "graphics/paletteman.h"
 
 #include "teenagent/scene.h"
 #include "teenagent/inventory.h"
@@ -48,7 +48,6 @@ Scene::Scene(TeenAgentEngine *vm) : _vm(vm), intro(false), _id(0), ons(0),
 	onEnabled = true;
 
 	memset(palette, 0, sizeof(palette));
-	background.pixels = 0;
 
 	FilePack varia;
 	varia.open("varia.res");
@@ -71,11 +70,13 @@ Scene::Scene(TeenAgentEngine *vm) : _vm(vm), intro(false), _id(0), ons(0),
 
 	varia.close();
 	loadObjectData();
+
+	_onsCount = 0;
+	_messageColor = 0;
 }
 
 Scene::~Scene() {
-	if (background.pixels)
-		background.free();
+	background.free();
 
 	delete[] ons;
 	ons = 0;
@@ -315,7 +316,7 @@ void Scene::loadOns() {
 	uint16 addr = _vm->res->dseg.get_word(dsAddr_onsAnimationTablePtr + (_id - 1) * 2);
 	debugC(0, kDebugScene, "ons index: %04x", addr);
 
-	onsCount = 0;
+	_onsCount = 0;
 	byte b;
 	byte onId[16];
 	while ((b = _vm->res->dseg.get_byte(addr)) != 0xff) {
@@ -324,15 +325,15 @@ void Scene::loadOns() {
 		if (b == 0)
 			continue;
 
-		onId[onsCount++] = b;
+		onId[_onsCount++] = b;
 	}
 
 	delete[] ons;
 	ons = NULL;
 
-	if (onsCount > 0) {
-		ons = new Surface[onsCount];
-		for (uint32 i = 0; i < onsCount; ++i) {
+	if (_onsCount > 0) {
+		ons = new Surface[_onsCount];
+		for (uint32 i = 0; i < _onsCount; ++i) {
 			Common::ScopedPtr<Common::SeekableReadStream> s(_vm->res->ons.getStream(onId[i]));
 			if (s) {
 				ons[i].load(*s, Surface::kTypeOns);
@@ -372,7 +373,7 @@ void Scene::init(int id, const Common::Point &pos) {
 	for (byte i = 0; i < 4; ++i)
 		customAnimation[i].free();
 
-	if (background.pixels == NULL)
+	if (background.getPixels() == NULL)
 		background.create(kScreenWidth, kScreenHeight, Graphics::PixelFormat::createFormatCLUT8());
 
 	warp(pos);
@@ -416,7 +417,7 @@ void Scene::init(int id, const Common::Point &pos) {
 	if (nowPlaying != _vm->res->dseg.get_byte(dsAddr_currentMusic))
 		_vm->music->load(_vm->res->dseg.get_byte(dsAddr_currentMusic));
 
-	_vm->_system->copyRectToScreen(background.pixels, background.pitch, 0, 0, background.w, background.h);
+	_vm->_system->copyRectToScreen(background.getPixels(), background.pitch, 0, 0, background.w, background.h);
 	setPalette(0);
 }
 
@@ -499,7 +500,7 @@ bool Scene::processEvent(const Common::Event &event) {
 				events.clear();
 				sounds.clear();
 				currentEvent.clear();
-				messageColor = textColorMark;
+				_messageColor = textColorMark;
 				for (int i = 0; i < 4; ++i)
 					customAnimation[i].free();
 				_vm->playMusic(4);
@@ -536,10 +537,13 @@ bool Scene::processEvent(const Common::Event &event) {
 		default:
 			break;
 		}
+		break;
 
 	default:
 		return false;
 	}
+
+	return false;
 }
 
 struct ZOrderCmp {
@@ -642,8 +646,8 @@ bool Scene::render(bool tickGame, bool tickMark, uint32 messageDelta) {
 			return true;
 		}
 
-		if (background.pixels && debugFeatures.feature[DebugFeatures::kShowBack]) {
-			_vm->_system->copyRectToScreen(background.pixels, background.pitch, 0, 0, background.w, background.h);
+		if (background.getPixels() && debugFeatures.feature[DebugFeatures::kShowBack]) {
+			_vm->_system->copyRectToScreen(background.getPixels(), background.pitch, 0, 0, background.w, background.h);
 		} else
 			_vm->_system->fillScreen(0);
 
@@ -652,7 +656,7 @@ bool Scene::render(bool tickGame, bool tickMark, uint32 messageDelta) {
 		bool gotAnyAnimation = false;
 
 		if (ons != NULL && debugFeatures.feature[DebugFeatures::kShowOns]) {
-			for (uint32 i = 0; i < onsCount; ++i) {
+			for (uint32 i = 0; i < _onsCount; ++i) {
 				Surface *s = ons + i;
 				if (s != NULL)
 					s->render(surface);
@@ -822,7 +826,7 @@ bool Scene::render(bool tickGame, bool tickMark, uint32 messageDelta) {
 			}
 
 			if (visible) {
-				_vm->res->font7.render(surface, messagePos.x, messagePos.y, message, messageColor);
+				_vm->res->font7.render(surface, messagePos.x, messagePos.y, message, _messageColor);
 				busy = true;
 			}
 		}
@@ -886,7 +890,7 @@ bool Scene::render(bool tickGame, bool tickMark, uint32 messageDelta) {
 		Sound &sound = *i;
 		if (sound.delay == 0) {
 			debugC(1, kDebugScene, "sound %u started", sound.id);
-			_vm->playSoundNow(sound.id);
+			_vm->playSoundNow(&_vm->res->sam_sam, sound.id);
 			i = sounds.erase(i);
 		} else {
 			sound.delay -= gameDelta;
@@ -1006,7 +1010,7 @@ bool Scene::processEventQueue() {
 					warning("no animation in slot %u", messageSlot);
 			}
 			messagePos = messagePosition(message, p);
-			messageColor = currentEvent.color;
+			_messageColor = currentEvent.color;
 
 			if (messageFirstFrame)
 				currentEvent.clear(); // async message, clearing event
@@ -1106,19 +1110,19 @@ bool Scene::processEventQueue() {
 
 		case SceneEvent::kEffect:
 			_vm->_system->delayMillis(80); // 2 vsyncs
-			_vm->_system->setShakePos(8);
+			_vm->_system->setShakePos(0, 8);
 			_vm->_system->updateScreen();
 
 			_vm->_system->delayMillis(80); // 2 vsyncs
-			_vm->_system->setShakePos(0);
+			_vm->_system->setShakePos(0, 0);
 			_vm->_system->updateScreen();
 
 			_vm->_system->delayMillis(80); // 2 vsyncs
-			_vm->_system->setShakePos(4);
+			_vm->_system->setShakePos(0, 4);
 			_vm->_system->updateScreen();
 
 			_vm->_system->delayMillis(80); // 2 vsyncs
-			_vm->_system->setShakePos(0);
+			_vm->_system->setShakePos(0, 0);
 			_vm->_system->updateScreen();
 
 			currentEvent.clear();
@@ -1154,7 +1158,7 @@ bool Scene::processEventQueue() {
 	}
 
 	if (events.empty()) {
-		messageColor = textColorMark;
+		_messageColor = textColorMark;
 		hideActor = false;
 	}
 
@@ -1233,7 +1237,7 @@ void Scene::displayMessage(const Common::String &str, byte color, const Common::
 	debugC(0, kDebugScene, "displayMessage: %s", str.c_str());
 	message = str;
 	messagePos = (pos.x | pos.y) ? pos : messagePosition(str, position);
-	messageColor = color;
+	_messageColor = color;
 	messageTimer = messageDuration(message);
 }
 
@@ -1252,7 +1256,7 @@ void Scene::clear() {
 void Scene::clearMessage() {
 	message.clear();
 	messageTimer = 0;
-	messageColor = textColorMark;
+	_messageColor = textColorMark;
 	messageFirstFrame = 0;
 	messageLastFrame = 0;
 	messageAnimation = NULL;
