@@ -26,15 +26,16 @@
 #include "ultima/nuvie/pathfinder/seek_path.h"
 #include "ultima/nuvie/pathfinder/actor_path_finder.h"
 #include "ultima/nuvie/pathfinder/party_path_finder.h"
+#include "ultima/nuvie/core/game.h"
+#include "ultima/nuvie/core/u6_objects.h"
 
 namespace Ultima {
 namespace Nuvie {
 
 using Std::vector;
 
-PartyPathFinder::PartyPathFinder(Party *p) {
+PartyPathFinder::PartyPathFinder(Party *p) : party(p) {
 	assert(p);
-	party = p;
 }
 
 PartyPathFinder::~PartyPathFinder() {
@@ -64,16 +65,26 @@ bool PartyPathFinder::is_at_target(uint32 p) {
 /* Is anyone in front of `member_num' adjacent to `from'?
  * (is_contiguous(member, member_loc) == "is member adjacent to another member
  * whose following position is lower-numbered?") */
-bool PartyPathFinder::is_contiguous(uint32 member_num, MapCoord from) {
+bool PartyPathFinder::is_contiguous(uint32 member_num, const MapCoord &from) {
+	const bool isU6 = Game::get_game()->get_game_type() == NUVIE_GAME_U6;
+	const Actor *inActor = get_member(member_num).actor;
+	const bool isMouse = isU6 && inActor->get_obj_n() == OBJ_U6_MOUSE;
+	bool retVal = false;
+
 	for (uint32 q = 0; q < member_num; q++) { // check lower-numbered members
-		Actor *actor = get_member(q).actor;
+		const Actor *actor = get_member(q).actor;
+		const bool otherIsMouse = isU6 && actor->get_obj_n() == OBJ_U6_MOUSE;
 		if (actor && actor->is_immobile() == true) continue;
 
 		MapCoord loc = party->get_location(q);
-		if (from.distance(loc) <= 1)
-			return true;
+		if (!isMouse && !otherIsMouse && from.distance(loc) == 0)
+			return false; // Do not allow stacking (except for Sherry)
+		if (from.distance(loc) <= 1) {
+			retVal = true;
+			continue; // Stay in the loop to check if other party members are on the same tile
+		}
 	}
-	return false;
+	return retVal;
 }
 
 /* Is member adjacent to another member whose following position is lower-numbered? */
@@ -99,7 +110,7 @@ void PartyPathFinder::get_forward_dir(sint8 &vec_x, sint8 &vec_y) {
 //    get_last_move(vec_x, vec_y);
 	vec_x = 0;
 	vec_y = 0;
-	uint8 dir = (get_leader() >= 0) ? get_member(get_leader()).actor->get_direction() : NUVIE_DIR_N;
+	NuvieDir dir = (get_leader() >= 0) ? get_member(get_leader()).actor->get_direction() : NUVIE_DIR_N;
 	if (dir == NUVIE_DIR_N)      {
 		vec_x = 0;
 		vec_y = -1;
@@ -334,7 +345,7 @@ bool PartyPathFinder::try_all_directions(uint32 p, MapCoord target_loc) {
  * distance to 'target'. (near to far)
  */
 vector<MapCoord>
-PartyPathFinder::get_neighbor_tiles(MapCoord &center, MapCoord &target) {
+PartyPathFinder::get_neighbor_tiles(const MapCoord &center, const MapCoord &target) {
 	sint8 rel_x = get_wrapped_rel_dir(target.x, center.x, target.z);
 	sint8 rel_y = get_wrapped_rel_dir(target.y, center.y, target.z);
 	vector<MapCoord> neighbors;
@@ -414,7 +425,7 @@ bool PartyPathFinder::move_member(uint32 member_num, sint16 relx, sint16 rely, b
 	MapCoord target(member_loc);
 	target = member_loc.abs_coords(relx, rely);
 	Actor *actor = get_member(member_num).actor;
-	ActorMoveFlags flags = ACTOR_IGNORE_MOVES;
+	ActorMoveFlags flags = ACTOR_IGNORE_MOVES | ACTOR_IGNORE_PARTY_MEMBERS;
 	if (!avoid_danger_tiles)
 		flags = flags | ACTOR_IGNORE_DANGER;
 
@@ -423,19 +434,22 @@ bool PartyPathFinder::move_member(uint32 member_num, sint16 relx, sint16 rely, b
 			actor->set_direction(relx, rely);
 			return true;
 		}
-		if (actor->get_error()->err == ACTOR_BLOCKED_BY_ACTOR) {
-			Actor *blocking_actor = actor->get_error()->blocking_actor;
-			sint8 blocking_member_num = -1;
-			if (blocking_actor)
-				blocking_member_num = party->get_member_num(blocking_actor);
-			if (blocking_member_num < sint32(member_num))
-				return false; // blocked by an actor not in the party
-			if (bump_member(uint32(blocking_member_num), member_num)
-			        && actor->move(target.x, target.y, target.z, flags | ACTOR_IGNORE_MOVES)) {
-				actor->set_direction(relx, rely);
-				return true;
-			}
-		}
+		// Block commented out: It seems that, at least in Ultima 6, party members do not
+		// bump each other, so we now ignore them via ACTOR_IGNORE_PARTY_MEMBERS.
+		// Instead, is_contiguous() prevents them from stacking.
+		// if (actor->get_error()->err == ACTOR_BLOCKED_BY_ACTOR) {
+		// 	const Actor *blocking_actor = actor->get_error()->blocking_actor;
+		// 	sint8 blocking_member_num = -1;
+		// 	if (blocking_actor)
+		// 		blocking_member_num = party->get_member_num(blocking_actor);
+		// 	if (blocking_member_num < sint32(member_num))
+		// 		return false; // blocked by an actor not in the party
+		// 	if (bump_member(uint32(blocking_member_num), member_num)
+		// 	        && actor->move(target.x, target.y, target.z, flags | ACTOR_IGNORE_MOVES)) {
+		// 		actor->set_direction(relx, rely);
+		// 		return true;
+		// 	}
+		// }
 	}
 	return false; // target is not contiguous, or move is blocked
 }
